<?php
/*
 Plugin Name: flowfact-wp-connector
 Description: Verbinden Sie Ihr Wordpress mit Ihrem FLOWFACT CRM.
 Version: 1.6.0
 Author: FLOWFACT
 Author URI: http://www.flowfact.de
 Text Domain: flowfact-wp-connector
 Plugin Slug: flowfact-wp-connector
 Domain Path: /languages
 License: GPL2
 License URI: https://www.gnu.org/licenses/gpl-2.0.html

 FLOWFACT WP Connector is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 2 of the License, or
 any later version.

 FLOWFACT WP Connector is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with FLOWFACT WP Connector. If not, see https://www.gnu.org/licenses/gpl-2.0.html.

 */


defined('ABSPATH') or die('No script kiddies please!');


class FlowfactSalesautomatConnector
{

    function __construct()
    {
        // load constants
        require_once(plugin_dir_path(__FILE__) . 'core/constants/default.php');
        require_once(plugin_dir_path(__FILE__) . 'core/constants/admin.php');
        require_once(plugin_dir_path(__FILE__) . 'core/constants/api.php');

        // load api
        require_once(plugin_dir_path(__FILE__) . 'core/api/api.php');


        
        // load admin panel
        require_once(plugin_dir_path(__FILE__) . 'core/admin/settings.php');

        // load composer packs.
        require_once(plugin_dir_path(__FILE__) . 'vendor/autoload.php');

        // create virtual page
        add_filter('the_posts', array($this, 'get_vitual_page_detect'), -1);
    }

    

    function plugin_activate()
    {

        // extend DB
        global $wpdb;
        $schemaharset_collate = $wpdb->get_charset_collate();
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        $sql = "CREATE TABLE " . $wpdb->prefix . "ff_entity_cache (
			  entityId varchar(80) NOT NULL,
			  schemaId varchar(80) NOT NULL,
			  created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			  json text NOT NULL,
			  PRIMARY KEY (entityId)
			  ) $schemaharset_collate;";
        dbDelta($sql);

        $sql = "CREATE TABLE " . $wpdb->prefix . "ff_schema_cache (
			  schemaId varchar(80) NOT NULL,
			  schemaName varchar(80) NOT NULL,
			  created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			  json text NOT NULL,
			  PRIMARY KEY (schemaId)
			  ) $schemaharset_collate;";
        dbDelta($sql);

        $sql = "CREATE TABLE " . $wpdb->prefix . "ff_general_cache (
			  name varchar(80) NOT NULL,
			  created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			  json text NOT NULL,
			  PRIMARY KEY (name)
			  ) $schemaharset_collate;";
        dbDelta($sql);

        $sql = "CREATE TABLE " . $wpdb->prefix . "ff_customer_cache (
			  id varchar(80) NOT NULL,
			  created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			  customer varchar(80) NOT NULL,
			  customerIp varchar(80) NOT NULL,
			  schemaId varchar(80) NOT NULL,
			  entityId varchar(80) NOT NULL,
			  value varchar(80) NOT NULL,
			  PRIMARY KEY (id)
			  ) $schemaharset_collate;";
        dbDelta($sql);

        // set rewrite
        set_transient('ff_flush', 1, 60);
    }

    function plugin_deactivate()
    {

        // cleanup DB
        global $wpdb;
        $schemaharset_collate = $wpdb->get_charset_collate();
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        $sql = "DROP TABLE IF EXISTS " . $wpdb->prefix . "ff_general_cache";
        $wpdb->query($sql);

        $sql = "DROP TABLE IF EXISTS " . $wpdb->prefix . "ff_schema_cache";
        $wpdb->query($sql);

        $sql = "DROP TABLE IF EXISTS " . $wpdb->prefix . "ff_entity_cache";
        $wpdb->query($sql);

        $sql = "DROP TABLE IF EXISTS " . $wpdb->prefix . "ff_customer_cache";
        $wpdb->query($sql);

        // flush rewrite rules
        flush_rewrite_rules();
    }


    function debug_to_console($data)
    {
        $output = $data;
        if (is_array($output))
            $output = implode(',', $output);
    }

    function get_vitual_page_detect($posts)
    {
        global $wp;
        global $wp_query;

        // get path
        $uriSegments = explode("/", parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH));

        foreach ($uriSegments as $key => $segment) {
            if ($segment == FF_PLUGIN_ROUTE) {

                if (!empty(FF_ESTATEVIEW_SEO_SLUG)) {
                    (!empty($uriSegments[$key])) ? $plugin = $uriSegments[$key] : "";
                    (!empty($uriSegments[$key + 1])) ? $module = $uriSegments[$key + 1] : "";
                    (!empty($uriSegments[$key + 2])) ? $slug = $uriSegments[$key + 2] : "";
                    (!empty($uriSegments[$key + 3])) ? $schema = $uriSegments[$key + 3] : "";
                    (!empty($uriSegments[$key + 4])) ? $identifier = $uriSegments[$key + 4] : "";
                } else {
                    (!empty($uriSegments[$key])) ? $plugin = $uriSegments[$key] : "";
                    (!empty($uriSegments[$key + 1])) ? $module = $uriSegments[$key + 1] : "";
                    (!empty($uriSegments[$key + 2])) ? $schema = $uriSegments[$key + 2] : "";
                    (!empty($uriSegments[$key + 3])) ? $identifier = $uriSegments[$key + 3] : "";
                }
            }
        }

        global $get_vitual_page_detect; // used to stop double loading
        $get_vitual_page_url = $_SERVER['REQUEST_URI']; // URL of the fake page


        if (!empty($_GET['console']) and $_GET['console'] == "true") {
            echo "<script>
                   console.log( 'current URL: " . parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH) . "' );
                   console.log( 'fake URL: " . $get_vitual_page_url . "' );
                   console.log( '- part plugin: " . $plugin . "' );
                   console.log( '- part module: " . $module . "' );
                   console.log( '- part slug: " . $slug . "' );
                   console.log( '- part schema: " . $schema . "' );
                   console.log( '- part identifier: " . $identifier . "' );
                </script>";
        }


        if (!empty($plugin) and $plugin == FF_PLUGIN_ROUTE) {

            // stop interferring with other $posts arrays on this page (only works if the sidebar is rendered *after* the main page)
            $get_vitual_page_detect = true;


            // create a fake virtual page
            $post = new stdClass;
            $post->post_author = 1;
            $post->post_name = $get_vitual_page_url;
            $post->guid = get_bloginfo('wpurl') . '/' . $get_vitual_page_url;

            if (!empty($plugin) && $plugin == FF_PLUGIN_ROUTE) {
                if ($module == FF_ESTATEVIEW_ROUTE) {

                    /* Substing added to allow to add time stamp to xml url to protect caching*/
                    if ((((!empty($slug) and substr($slug, -4) === ".xml" and substr($slug, 0, 7) === "sitemap") or (!empty($schema) and substr($schema, -4) === ".xml" and substr($schema, 0, 7) === "sitemap")) or ((!empty($slug) and substr($slug, -4) === ".txt" and substr($slug, 0, 7) === "sitemap") or (!empty($schema) and substr($schema, -4) === ".txt" and substr($schema, 0, 7) === "sitemap"))) and FF_ESTATEVIEW_SEO == "on") {
                        if (substr($slug, -4) === ".xml" or substr($schema, -4) === ".xml") {
                            $type = "xml";
                        } else if (substr($slug, -4) === ".txt" or substr($schema, -4) === ".txt") {
                            $type = "txt";
                        } else {
                            $type = "xml";
                        }

                        $FFestateViewCore = new FFestateViewCore();
                        $data = $FFestateViewCore->get_estate_sitemap($type);
                        header("Content-type: text/plain");
                        echo $data['content'];
                        die;

                    } elseif (!empty($identifier)) {
                        $FFestateViewCore = new FFestateViewCore();
                        $data = $FFestateViewCore->get_estate_details($schema, $identifier);

                        $post->post_title = $data['title'];
                        $post->post_content = $data['content'];
                    } else {
                        $FFestateViewCore = new FFestateViewCore();
                        $data = $FFestateViewCore->get_estate_overview();
                        $post->post_title = $data['title'];
                        $post->post_content = $data['content'];

                    }
                } elseif ($module == FF_ESTATEREFERENCE_ROUTE) {
                    $FFestateReferenceCore = new FFestateReferenceCore();
                    $data = $FFestateReferenceCore->get_estate_reference_overview();
                    $post->post_title = $data['title'];
                    $post->post_content = $data['content'];
                } elseif ($module == FF_VALUATION_ROUTE) {
                    $FFvaluationCore = new FFvaluationCore();
                    $data = $FFvaluationCore->get_overview();

                    $post->post_title = $data['title'];
                    $post->post_content = $data['content'];

                } elseif ($module == FF_VALUATIONMASTER_ROUTE) {
                    if ((!empty($schema) and $schema == "report") or (!empty($slug) and $slug == "report")) {
                        $FFvaluationMasterCore = new FFvaluationMasterCore();

                        // here we gonna inject views
                        $data = $FFvaluationMasterCore->get_results_by_url();

                        $post->post_title = $data['title'];
                        $post->post_content = $data['content'];
                    } else if ((!empty($schema) and $schema == "call") or (!empty($slug) and $slug == "call")) {
                        $FFvaluationMasterCore = new FFvaluationMasterCore();
                        $data = $FFvaluationMasterCore->send_callback();


                    } else if ((!empty($schema) and $schema == "store") or (!empty($slug) and $slug == "store")) {
                        $FFvaluationMasterCore = new FFvaluationMasterCore();
                        if (!isset($GLOBALS['valuationMasterSent']) || !$GLOBALS['valuationMasterSent']) {
                            $data = $FFvaluationMasterCore->get_result();

                            $post->post_title = $data['title'];
                            $post->post_content = $data['content'];
                        }
                    } else {
                        $FFvaluationMasterCore = new FFvaluationMasterCore();
                        $data = $FFvaluationMasterCore->get_overview();

                        $post->post_title = $data['title'];
                        $post->post_content = $data['content'];
                    }
                } elseif ($module == FF_OWNERREPORT_ROUTE) {
                    $FFownerReportCore = new FFownerReportCore();
                    if (session_id() == '') {
                        session_start();
                    }

                    if (!empty($_POST['action']) && $_POST['action'] == 'ffOwnerReport_objecTracking') {
                        $login_attempt = true;
                        $code = $_POST['ffOwnerReport_objecTracking'];
                        $FFownerReportCore->handle_login($code);
                    }

                    if (!$FFownerReportCore->is_user_logged_in()) {
                        $data = $FFownerReportCore->get_login_page();
                        $post->post_title = $data['title'];
                        $post->post_content = $data['content'];
                    } else {
                        if (!empty($schema) && $schema == 'logout') {
                            $logged_out['logged_out'] = $FFownerReportCore->logout_user();
                            $data = $FFownerReportCore->get_login_page(array('logged_out' => $logged_out));
                            $post->post_title = $data['title'];
                            $post->post_content = $data['content'];
                        } elseif (empty($schema) || ($schema == 'estate' && empty($identifier))) {
                            $data = $FFownerReportCore->get_report_or_list();
                            $post->post_title = $data['title'];
                            $post->post_content = $data['content'];
                        } elseif (!empty($identifier)) {
                            $data = $FFownerReportCore->get_owner_report($identifier);
                            $post->post_title = $data['title'];
                            $post->post_content = $data['content'];
                        }
                    }
                } else {
                    global $wp_query;
                    $wp_query->set_404();
                    status_header(404);
                    get_template_part(404);
                    exit();
                }
            }

            $post->ID = -999;
            $post->post_type = FF_POST_TYPE;
            $post->post_status = 'static';
            $post->comment_status = 'closed';
            $post->ping_status = 'open';
            $post->comment_count = 0;
            $post->post_date = current_time('mysql');
            $post->post_date_gmt = current_time('mysql', 1);
            $posts = NULL;
            $posts[] = $post;

            // make wpQuery believe this is a real page too
            if (FF_POST_TYPE == 'page') {
                $wp_query->is_page = true;
                $wp_query->is_single = false;
            } else {
                $wp_query->is_page = false;
                $wp_query->is_single = true;
            }

            $wp_query->is_singular = true;
            $wp_query->is_home = false;
            $wp_query->is_archive = false;
            $wp_query->is_category = false;
            unset($wp_query->query["error"]);
            $wp_query->query_vars["error"] = "";
            $wp_query->is_404 = false;
        }

        remove_filter('the_content', 'wpautop');
        remove_filter('the_excerpt', 'wpautop');


        if (!empty($_GET["iframe"]) and $_GET["iframe"] == "1") {
            wp_head();
            echo $post->post_content;
            wp_footer();
            die;

        } else {
            return $posts;
        }
    }


    function init_modules()
    {


        // init widgets FFestateViewWidget
        add_action('widgets_init', function () {
            require_once(plugin_dir_path(__FILE__) . 'modules/estateView/config.php');
            require_once(plugin_dir_path(__FILE__) . 'modules/estateView/widget.php');
            register_widget('FFestateViewWidget');
        });


        // init widgets FFestateReferenceWidget
        add_action('widgets_init', function () {
            require_once(plugin_dir_path(__FILE__) . 'modules/estateReference/config.php');
            require_once(plugin_dir_path(__FILE__) . 'modules/estateReference/widget.php');
            register_widget('FFestateReferenceWidget');
        });

        // init widgets FFvaluationWidget
        add_action('widgets_init', function () {
            require_once(plugin_dir_path(__FILE__) . 'modules/valuation/config.php');
            require_once(plugin_dir_path(__FILE__) . 'modules/valuation/widget.php');
            register_widget('FFvaluationWidget');
        });

        // init widgets FFestateViewWidget
        add_action('widgets_init', function () {
            require_once(plugin_dir_path(__FILE__) . 'modules/teamOverview/config.php');
            require_once(plugin_dir_path(__FILE__) . 'modules/teamOverview/widget.php');
            register_widget('FFteamOverviewWidget');
        });

        // init widgets FFFormIntegrationWidget
        add_action('widgets_init', function () {
            require_once(plugin_dir_path(__FILE__) . 'modules/formIntegration/config.php');
            require_once(plugin_dir_path(__FILE__) . 'modules/formIntegration/widget.php');
            register_widget('FFFormIntegrationWidget');
        });

        // init widgets FFcompanyPlaceholder
        require_once(plugin_dir_path(__FILE__) . 'modules/companyPlaceholder/config.php');
        require_once(plugin_dir_path(__FILE__) . 'modules/companyPlaceholder/core.php');

        // init widgets FFvaluationMasterWidget
        add_action('widgets_init', function () {
            require_once(plugin_dir_path(__FILE__) . 'modules/valuationMaster/config.php');
            require_once(plugin_dir_path(__FILE__) . 'modules/valuationMaster/widget.php');
            register_widget('FFvaluationMasterWidget');
        });

        // init widgets FFownerReportWidget
        add_action('widgets_init', function () {
            require_once(plugin_dir_path(__FILE__) . 'modules/ownerReport/config.php');
            require_once(plugin_dir_path(__FILE__) . 'modules/ownerReport/widget.php');
            register_widget('FFownerReportWidget');
        });

    }
}

if (class_exists('FlowfactSalesautomatConnector')) {
    $flowfactSalesautomatConnector = new FlowfactSalesautomatConnector();
    $flowfactSalesautomatConnector->init_modules();
}

// activate
register_activation_hook(__FILE__, array($flowfactSalesautomatConnector, 'plugin_activate'));

// deactivate
register_deactivation_hook(__FILE__, array($flowfactSalesautomatConnector, 'plugin_deactivate'));
